<?php

namespace Drupal\azure_smtp_oauth\Controller;

use Drupal\azure_smtp_oauth\Service\RefreshTokenService;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use TheNetworg\OAuth2\Client\Provider\Azure;

/**
 * Controller for handling OAuth authorization.
 */
class GenerateOauthCodeController extends ControllerBase {

  /**
   * The Refresh Token Service .
   *
   * @var \Drupal\azure_smtp_oauth\Service\RefreshTokenService
   */
  protected $refreshTokenService;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs Refreshtoken objects.
   *
   * @param Drupal\azure_smtp_oauth\Service\RefreshTokenService $refreshTokenService
   *   The RefreshtokenService object.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The Configfactory object.
   */
  public function __construct(RefreshTokenService $refreshTokenService, ConfigFactoryInterface $config_factory) {
    $this->refreshTokenService = $refreshTokenService;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
    $container->get('azure_smtp_oauth.refresh_token_service'),
    $container->get('config.factory'),
    );
  }

  /**
   * Initiates the OAuth authorization flow.
   */
  public function getAuthorizationCode(Request $request) {
    $config = $this->configFactory->getEditable('azure_smtp_oauth.settings');
    try {
      if (!empty($config->get('oauth_client_id'))) {
        $clientId = trim($config->get('oauth_client_id'));
      }
      if (!empty($config->get('oauth_client_secret'))) {
        $clientSecret = trim($config->get('oauth_client_secret'));
      }
      if (!empty($config->get('oauth_redirect_uri'))) {
        $redirectUri = trim($config->get('oauth_redirect_uri'));
      }
    }
    catch (\Exception $e) {
      $this->logger->error('Error retrieving configuration values: ' . $e->getMessage());
      return 'Error while retrieving configuration values';
    }

    $provider = new Azure([
      'clientId' => $clientId,
      'clientSecret' => $clientSecret,
      'redirectUri' => $redirectUri,
      'response_type' => 'code',
      'scope' => ['openid', 'profile', 'email', 'offline_access'],
    ]);

    $code = $request->query->get('code');

    if (empty($code)) {
      // If we don't have an authorization code, then get one.
      $authorizationUrl = $provider->getAuthorizationUrl([
        'scope' => ['openid', 'profile', 'email', 'offline_access'],
      ]);
      header('Location: ' . $authorizationUrl);
      exit;
    }
    else {
      $this->refreshTokenService->getRefreshToken($code);
    }
    return ['#markup' => 'Refresh token is generated. Please refresh the configuration page once again.'];

  }

}
