<?php

namespace Drupal\azure_smtp_oauth\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use TheNetworg\OAuth2\Client\Provider\Azure;

/**
 * Refresh token to get authorization code.
 */
class RefreshTokenService {

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * Constructs a RefreshTokenService service.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The Configfactory object.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger object.
   */
  public function __construct(ConfigFactoryInterface $configFactory, LoggerChannelFactoryInterface $logger) {
    $this->configFactory = $configFactory;
    $this->logger = $logger;
  }

  /**
   * Function to get Refresh token for oAuth.
   */
  public function getRefreshToken($code) {
    $config = $this->configFactory->getEditable('azure_smtp_oauth.settings');

    try {
      if (!empty($config->get('oauth_client_id'))) {
        $clientId = trim($config->get('oauth_client_id'));
      }
      if (!empty($config->get('oauth_client_secret'))) {
        $clientSecret = trim($config->get('oauth_client_secret'));
      }
      if (!empty($config->get('oauth_redirect_uri'))) {
        $redirectUri = trim($config->get('oauth_redirect_uri'));
      }

      // To check if refresh token is present or not.
      if (!empty($config->get('oauth_refresh_token'))) {
        $oauthRefreshToken = trim($config->get('oauth_refresh_token'));
        return $oauthRefreshToken;
      }
      else {
        $provider = new Azure([
          'clientId' => $clientId,
          'clientSecret' => $clientSecret,
          'redirectUri' => $redirectUri,
          'response_type' => 'code',
          'scope' => ['openid', 'profile', 'email', 'offline_access'],
        ]);
        $accessToken = $provider->getAccessToken(
          'authorization_code',
          [
            'code' => $code,
          ]
        );
        $refreshToken = $accessToken->getRefreshToken();

        $config->set('oauth_refresh_token', $refreshToken)->save();
        return $refreshToken;
      }
    }
    catch (\Exception $e) {
      $this->logger->error('Error retrieving configuration values: ' . $e->getMessage());
      return FALSE;
    }
  }

}
